@@ -158,10 +158,24 @@
 
 #define E100_DEFAULT_TCB   MAX_TCB
 #define E100_MIN_TCB       2*TX_FRAME_CNT + 3 /* make room for at least 2 interrupts */
+
+#ifdef __ia64__
+ /* We can't use too many DMAble buffers on IA64 machines with >4 GB mem*/
+#define E100_MAX_TCB       64
+#else
 #define E100_MAX_TCB       1024
+#endif /*  __ia64__ */
+
 #define E100_DEFAULT_RFD   MAX_RFD
 #define E100_MIN_RFD       8
+
+#ifdef __ia64__
+ /* We can't use too many DMAble buffers on IA64 machines with >4 GB mem*/
+#define E100_MAX_RFD       64  
+#else
 #define E100_MAX_RFD       1024
+#endif /*  __ia64__ */
+
 #define E100_DEFAULT_XSUM  TRUE
 
 #define TX_CONG_DFLT   0       /* congestion enable flag for National TX Phy */
@@ -265,11 +279,33 @@
 #include <asm/io.h>
 
 
+#if (( LINUX_VERSION_CODE < KERNEL_VERSION(2,2,18) ) || \
+     ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0) ) )
+typedef unsigned long dma_addr_t;
+#endif
 
+#define PCI_DMA_TODEVICE   1
+#define PCI_DMA_FROMDEVICE 2
+
+extern inline void *pci_alloc_consistent (struct pci_dev *dev, 
+                                          size_t size, 
+                                          dma_addr_t *dma_handle) {
+    void *vaddr = kmalloc(size, GFP_ATOMIC);
+    if(vaddr != NULL) {
+        *dma_handle = virt_to_bus(vaddr);
+    }
+    return vaddr; 
+}
+#define pci_dma_sync_single(dev,dma_handle,size,direction)   do{} while(0)
 #define pci_dma_supported(dev, addr_mask)                    (1)
 #define pci_free_consistent(dev, size, cpu_addr, dma_handle) kfree(cpu_addr)
 #define pci_map_single(dev, addr, size, direction)           virt_to_bus(addr)
 #define pci_unmap_single(dev, dma_handle, size, direction)   do{} while(0)
+
+#define pci_resource_len(a,b)        (128 * 1024)
+
+static inline int request_mem_region(long int addr, ...) { return 1; }
+static inline int release_mem_region(long int addr, ...) { return 0; }
 #endif
 
 /*****************************************************************************
@@ -1098,7 +1134,7 @@ typedef struct net_device_stats net_device_stats_t;
 
 /* PHY 100 Extended Register 0 bit definitions*/
 #define PHY_100_ER0_FDX_INDIC       BIT_0   /* 1 = FDX, 0 = half duplex */
-#define PHY_100_ER0_SPEED_INDIC     BIT_1   /* 1 = 100mbs, 0= 10mbs */
+#define PHY_100_ER0_SPEED_INDIC     BIT_1   /* 1 = 100Mbps, 0= 10Mbps */
 #define PHY_100_ER0_WAKE_UP         BIT_2   /* Wake up DAC */
 #define PHY_100_ER0_RESERVED        BIT_3_4   /* Reserved */
 #define PHY_100_ER0_REV_CNTRL       BIT_5_7   /* Revsion control (A step = 000) */
@@ -1123,7 +1159,7 @@ typedef struct net_device_stats net_device_stats_t;
 #define NSC_TX_CONG_ENABLE          BIT_8   /* Enables congestion control */
 
 /* National Semiconductor TX phy speed indication register bit definitions*/
-#define NSC_TX_SPD_INDC_SPEED       BIT_6   /* 0 = 100mb, 1=10mb */
+#define NSC_TX_SPD_INDC_SPEED       BIT_6   /* 0 = 100Mbps, 1=10Mbps */
 
 /* Return status for xmit bufs */
 #define E_NOBUFS  0x10           /* no more buffers */
@@ -1380,8 +1416,11 @@ typedef struct net_device_stats net_dev_stats_t;
 #define get_pci_bus(X)          ((X)>>8)
 #define mk_pci_dev_id(X,Y)      (((X)<<8)|((Y)&PCI_DEV_NO))
 
+#define E100_MIN(X,Y)         ( (X) < (Y)? (X):(Y) )
 #define RFD_POINTER(X,Y)      ((rfd_t *) (((unsigned char *)((X)->data))-((Y)->rfd_size)))
-
+#define SKB_RFD_COMPLETE(X,Y) ((RFD_POINTER((X),(Y)))->rfd_header.cb_status)
+#define NEXT_SKB(X,Y)         ((RFD_POINTER((X),(Y)))->next)   
+#define GET_SKB_DMA_ADDR(skb,bddp) ((RFD_POINTER(skb,bddp))->dma_addr)
 /* ====================================================================== */
 /*                              82557                                     */
 /* ====================================================================== */
@@ -1678,35 +1717,36 @@ typedef struct _tcb_ipcb_t
 typedef struct _tcb_gen_t
 {
     eth_tx_buffer_t tcb_data;     /* Data buffer in TCB */
-    ulong_t pad0;                 /* filler */
-    ulong_t pad1;                 /* filler */
-    ulong_t pad2;                 /* filler */
-    ulong_t pad3;                 /* filler */
+    uint32_t pad0;                 /* filler */
+    uint32_t pad1;                 /* filler */
+    uint32_t pad2;                 /* filler */
+    uint32_t pad3;                 /* filler */
 } tcb_gen_t __attribute__ ((__packed__));
 
 /* Transmit Command Block (TCB)*/
 struct _tcb_t
 {
-    cb_header_t tcb_hdr;
-    uint32_t tcb_tbd_ptr;           /* TBD address */
-    uint16_t tcb_cnt;              /* Data Bytes In TCB past header */
-    uint8_t tcb_thrshld;           /* TX Threshold for FIFO Extender */
-    uint8_t tcb_tbd_num;
+  cb_header_t tcb_hdr;
+  uint32_t tcb_tbd_ptr;           /* TBD address */
+  uint16_t tcb_cnt;              /* Data Bytes In TCB past header */
+  uint8_t tcb_thrshld;           /* TX Threshold for FIFO Extender */
+  uint8_t tcb_tbd_num;
 
-    union {
-        tcb_ipcb_t ipcb;                  /* d102 ipcb fields */
-        tcb_ext_t tcb_ext;        /* 82558 extended TCB fields */
-        tcb_gen_t tcb_gen;        /* Generic 82557 fields */
-    } tcbu;
+  union {
+    tcb_ipcb_t ipcb;                  /* d102 ipcb fields */
+    tcb_ext_t tcb_ext;        /* 82558 extended TCB fields */
+    tcb_gen_t tcb_gen;        /* Generic 82557 fields */
+  } tcbu;
 
-    /* From here onward we can dump anything we want as long as the
-     * size of the total structure is a multiple of a paragraph
-     * boundary ( i.e. -16 bit aligned ).
-     */
-    sk_buff_t *tcb_skb;           /* the associated socket buffer */
-    int tcb_cbflag;              /* set if using the coalesce buffer */
-    int tcb_msgsz;
-    uint32_t tcb_paddr;           /* phys addr of the TCB */
+  /* From here onward we can dump anything we want as long as the
+   * size of the total structure is a multiple of a paragraph
+   * boundary ( i.e. -16 bit aligned ).
+   */
+  sk_buff_t *tcb_skb;           /* the associated socket buffer */
+  int tcb_cbflag;              /* set if using the coalesce buffer */
+  int tcb_msgsz;
+  uint32_t dma_data_addr;
+  uint32_t tcb_paddr;           /* phys addr of the TCB */
 } __attribute__ ((__packed__));
 
 
@@ -1746,6 +1786,7 @@ struct _rfd_t
     sk_buff_t *prev;              /* skb containing previous RFD */
     sk_buff_t *next;              /* skb containing next RFD */
     device_t  *dev;               /* identify which dev this RFD belongs to */
+  dma_addr_t dma_addr;
 } __attribute__ ((__packed__));
 
 #ifndef _RFD_T_
@@ -1790,20 +1831,20 @@ typedef struct _bdd_t
     size_t mem_size;              /* Memory size from devmem_size */
     pscb_t scbp;                 /* memory mapped ptr to 82557 scb */
     pself_test_t pselftest;        /* pointer to self test area */
-    paddr_t selftest_paddr;        /* phys addr of selftest */
+    dma_addr_t selftest_paddr;        /* phys addr of selftest */
     perr_cntr_t pstats_counters; /* pointer to stats table */
-    paddr_t stat_cnt_paddr;        /* phys addr of stat counter area */
+    dma_addr_t stat_cnt_paddr;        /* phys addr of stat counter area */
     pdump_area_t pdump_area;     /* pointer to 82557 reg. dump area */
-    paddr_t dump_paddr;           /* phys addr of dump area */
+    dma_addr_t dump_paddr;           /* phys addr of dump area */
     pnxmit_cb_t pntcb;           /* pointer to non xmit tcb */
-    paddr_t nontx_paddr;           /* phys addr of  non-tx tcb */
+    dma_addr_t nontx_paddr;           /* phys addr of  non-tx tcb */
     perr_stats_t perr_stats;     /* ptr to error statictics results */
-    paddr_t estat_paddr;           /* phys addr of  err stat results */
+    dma_addr_t estat_paddr;           /* phys addr of  err stat results */
     buf_pool_t tcb_pool;
-    paddr_t tcb_paddr;           /* phys addr of start of TCBs */
+    dma_addr_t tcb_paddr;           /* phys addr of start of TCBs */
     void *tcb_pool_base;
     buf_pool_t tbd_pool;
-    paddr_t tbd_paddr;           /* phys addr of start of TBDs */
+    dma_addr_t tbd_paddr;           /* phys addr of start of TBDs */
     sk_buff_t *rfd_head;
     sk_buff_t *rfd_tail;
     int skb_req;
@@ -1905,6 +1946,7 @@ struct bdconfig
     /* these are linux only */
     struct sk_buff *last_mp_fail;
     device_t *device;
+  pci_dev_t *ppci_dev;
     uint8_t pci_bus;
     uint8_t pci_dev_fun;
     uint16_t vendor;
@@ -2907,3 +2949,4 @@ extern e100_vendor_info_t e100_vendor_info_array[];
 #endif
 
 
+
